@@ -399,22 +399,24 @@ class MMapCache(BaseCache):
     def _makefile(self):
         import tempfile
         import mmap
-        from builtins import open
+
+        if self.size == 0:
+            return bytearray()
+
         # posix version
         if self.location is None or not os.path.exists(self.location):
             if self.location is None:
                 fd = tempfile.TemporaryFile()
+                self.blocks = set()
             else:
-                fd = open(self.location, 'wb+')
+                fd = io.open(self.location, 'wb+')
             fd.seek(self.size - 1)
             fd.write(b'1')
             fd.flush()
         else:
-            fd = open(self.location, 'rb+')
-        self._file = fd
+            fd = io.open(self.location, 'rb+')
 
-        f_no = fd.fileno()
-        return mmap.mmap(f_no, self.size)
+        return mmap.mmap(fd.fileno(), self.size)
 
     def _fetch(self, start, end):
         start_block = start // self.blocksize
@@ -432,6 +434,17 @@ class MMapCache(BaseCache):
 
         return self.cache[start:end]
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        # Remove the unpicklable entries.
+        del state['cache']
+        return state
+
+    def __setstate__(self, state):
+        # Restore instance attributes
+        self.__dict__.update(state)
+        self.cache = self._makefile()
+
 
 class BytesCache(BaseCache):
     """Cache which holds data in a in-memory bytes object
@@ -459,13 +472,20 @@ class BytesCache(BaseCache):
     def _fetch(self, start, end):
         # TODO: only set start/end after fetch, in case it fails?
         # is this where retry logic might go?
-        if self.start is None and self.end is None:
+        if self.blocksize:
+            bend = min(self.size, end + self.blocksize)
+        else:
+            bend = end
+        if bend == start:
+            return b""
+        if ((self.start is None or start < self.start) and
+                (self.end is None or end > self.end)):
             # First read
-            self.cache = self.fetcher(start, end + self.blocksize)
+            self.cache = self.fetcher(start, bend)
             self.start = start
         elif start < self.start:
             if self.end - end > self.blocksize:
-                self.cache = self.fetcher(start, end + self.blocksize)
+                self.cache = self.fetcher(start, bend)
                 self.start = start
             else:
                 new = self.fetcher(start, self.start)
@@ -475,10 +495,10 @@ class BytesCache(BaseCache):
             if self.end > self.size:
                 pass
             elif end - self.end > self.blocksize:
-                self.cache = self.fetcher(start, end + self.blocksize)
+                self.cache = self.fetcher(start, bend)
                 self.start = start
             else:
-                new = self.fetcher(self.end, end + self.blocksize)
+                new = self.fetcher(self.end, bend)
                 self.cache = self.cache + new
 
         self.end = self.start + len(self.cache)
