@@ -67,6 +67,9 @@ class PyArrowHDFS(AbstractFileSystem):
         if detail:
             for p in out:
                 p['type'] = p['kind']
+                p["name"] = self._strip_protocol(p["name"])
+        else:
+            out = [self._strip_protocol(p) for p in out]
         return out
 
     @staticmethod
