@@ -263,7 +263,8 @@ class AbstractFileSystem(up):
         but must be consistent across implementations.
         Must include:
         - full path to the entry (without protocol)
-        - size of the entry, in bytes
+        - size of the entry, in bytes. If the value cannot be determined, will
+          be ``None``.
         - type of entry, "file", "directory" or other
 
         Additional information
@@ -357,7 +358,7 @@ class AbstractFileSystem(up):
                                      **kwargs):
                     yield res
 
-    def find(self, path, maxdepth=None, **kwargs):
+    def find(self, path, maxdepth=None, withdirs=False, **kwargs):
         """List all files below path.
 
         Like posix ``find`` command without conditions
@@ -366,13 +367,18 @@ class AbstractFileSystem(up):
         ----------
         maxdepth: int or None
             If not None, the maximum number of levels to descend
+        withdirs: bool
+            Whether to include directory paths in the output. This is True
+            when used by glob, but users usually only want files.
         kwargs are passed to ``ls``.
         """
         # TODO: allow equivalent of -name parameter
         out = []
-        for path, _, files in self.walk(path, maxdepth, **kwargs):
+        for path, dirs, files in self.walk(path, maxdepth, **kwargs):
+            if withdirs:
+                files += dirs
             for name in files:
-                if name:
+                if name and name not in out:
                     out.append('/'.join([path.rstrip('/'), name])
                                if path else name)
         if self.isfile(path) and path not in out:
@@ -441,9 +447,18 @@ class AbstractFileSystem(up):
         else:
             root = ''
             depth = 20 if "**" in path else 1
-        allpaths = self.find(root, maxdepth=depth, **kwargs)
-        pattern = "^" + path.replace('.', r'\.').replace('//', '/').rstrip(
-            '/').replace('?', '.') + "$"
+        allpaths = self.find(root, maxdepth=depth, withdirs=True, **kwargs)
+        pattern = "^" + (
+            path.replace('\\', r'\\')
+            .replace('.', r'\.')
+            .replace('+', r'\+')
+            .replace('//', '/')
+            .replace('(', r'\(')
+            .replace(')', r'\)')
+            .replace('|', r'\|')
+            .rstrip('/')
+            .replace('?', '.')
+        ) + "$"
         pattern = re.sub('[*]{2}', '=PLACEHOLDER=', pattern)
         pattern = re.sub('[*]', '[^/]*', pattern)
         pattern = re.compile(pattern.replace("=PLACEHOLDER=", '.*'))
@@ -468,6 +483,9 @@ class AbstractFileSystem(up):
         The default implementation should calls ls and could be overridden by a
         shortcut. kwargs are passed on to ```ls()``.
 
+        Some file systems might not be able to measure the file's size, in
+        which case, the returned dict will include ``'size': None``.
+
         Returns
         -------
         dict with keys: name (full path in the FS), size (in bytes), type (file,
@@ -482,6 +500,8 @@ class AbstractFileSystem(up):
         path = path.rstrip('/')
         out1 = [o for o in out if o['name'].rstrip('/') == path]
         if len(out1) == 1:
+            if "size" not in out1[0]:
+                out1[0]['size'] = None
             return out1[0]
         elif len(out1) > 1 or out:
             return {'name': path, 'size': 0, 'type': 'directory'}
@@ -503,7 +523,7 @@ class AbstractFileSystem(up):
 
     def size(self, path):
         """Size in bytes of file"""
-        return self.info(path)['size']
+        return self.info(path).get('size', None)
 
     def isdir(self, path):
         """Is this entry directory-like?"""
